#!/usr/bin/env python
import sys
from pysilico_server.process_monitor.runner import Runner


__version__ = "$Id: tipico_run_process_monitor.py 26 2018-01-26 19:06:25Z lbusoni $"


if __name__ == '__main__':
    runner= Runner()
    sys.exit(runner.start(sys.argv))



#!/usr/bin/env python
import sys
from pysilico_server.camera_controller.runner import Runner

__version__ = "$Id: tipico_run_mirror_controller.py 26 2018-01-26 19:06:25Z lbusoni $"



def main():
    runner= Runner()
    sys.exit(runner.start(sys.argv))


if __name__ == '__main__':
    main()



#!/usr/bin/env python
import unittest
import numpy as np
import logging
from pysilico_server.devices.simulated_auxiliary_camera import \
    SimulatedAuxiliaryCamera
from plico.utils.logger import Logger

__version__ = "$Id: simulated_auxiliary_camera_test.py 271 2017-04-28 16:49:02Z lbusoni $"


class SimulatedAuxiliaryCameraTest(unittest.TestCase):

    def setUp(self):
        self._setUpLogging()
        self._camera = SimulatedAuxiliaryCamera()


    def tearDown(self):
        self._camera.raiseExceptionOnDeinitialize(False)
        self._camera.deinitialize()


    def _setUpLogging(self):
        FORMAT='%(asctime)s %(levelname)s %(message)s'
        logging.basicConfig(level=logging.DEBUG, format=FORMAT)
        self._logger= Logger.of(self.__class__.__name__)


    def testReadFrameDimensionsAndType(self):
        self._camera.setBinning(1)
        f = self._camera.readFrame()
        self.assertEqual(f.shape, (self._camera.SENSOR_H,
                                   self._camera.SENSOR_W))
        self.assertEqual(f.dtype, self._camera.DTYPE)


    def testSetNoise(self):
        self._camera.setNoiseInCount(100.)
        f = self._camera.readFrame()
        self.assertAlmostEqual(100, np.std(f[0:100, 0:100]), delta=3)


    def testFluxIsProportionalToExposureTime(self):
        self._camera.setNoiseInCount(0)
        self._camera.setExposureTime(0.1)
        ima1ms= self._camera.readFrame()
        self._camera.setExposureTime(0.25)
        ima2ms= self._camera.readFrame()
        self.assertAlmostEqual(2.5* ima1ms.max(),
                               ima2ms.max(),
                               delta=3)


    def testFrameIsSaturatedIfFluxIsTooHigh(self):
        self._camera.setTotalFluxPerMilliSecond(1e9)
        frame= self._camera.readFrame()
        self.assertEqual(self._camera.MAX_VALUE,
                         frame.max())



if __name__ == "__main__":
    unittest.main()




__version__ = "$Id: constants.py 31 2018-01-27 10:47:29Z lbusoni $"


class Constants:
    APP_NAME= "inaf.arcetri.ao.pysilico_server"
    APP_AUTHOR= "INAF Arcetri Adaptive Optics"
    THIS_PACKAGE= 'pysilico_server'

    PROCESS_MONITOR_CONFIG_SECTION= 'processMonitor'
    SERVER_1_CONFIG_SECTION= 'avt1'
    SERVER_2_CONFIG_SECTION= 'avt2'

    # TODO: must be the same of console_scripts in setup.py
    START_PROCESS_NAME= 'pysilico_start'
    STOP_PROCESS_NAME= 'pysilico_stop'
    KILL_ALL_PROCESS_NAME= 'pysilico_kill_all'
    SERVER_1_PROCESS_NAME= 'pysilico_server_1'
    SERVER_2_PROCESS_NAME= 'pysilico_server_2'



import os
import time
from plico.utils.base_runner import BaseRunner
from pysilico_server.devices.simulated_camera import \
    SimulatedPyramidWfsCamera
from pysilico_server.devices.simulated_auxiliary_camera import \
    SimulatedAuxiliaryCamera
from plico.utils.logger import Logger
from plico.utils.control_loop import FaultTolerantControlLoop
from plico.utils.decorator import override
from pysilico_server.camera_controller.camera_controller import \
    CameraController
from plico.rpc.zmq_ports import ZmqPorts
import functools


def ContextWrapper():
    def wrapperFunc(f):
        @functools.wraps(f)
        def wrapper_vimba(self, *args, **kwds):
            from vimba import Vimba
            with Vimba.get_instance():
                with self._vimbacamera:
                    return f(self, *args, **kwds)

        def wrapper_generic(self, *args, **kwds):
            return f(self, *args, **kwds)

        def wrapper(self, *args, **kwds):
            if self._use_vimba_wrapper:
                return wrapper_vimba(self, *args, **kwds)
            else:
                return wrapper_generic(self, *args, **kwds)
        return wrapper

    return wrapperFunc


class Runner(BaseRunner):

    RUNNING_MESSAGE = "Camera controller is running."

    def __init__(self):
        BaseRunner.__init__(self)
        self._use_vimba_wrapper = False

    def _createCameraDevice(self):
        cameraDeviceSection = self.configuration.getValue(
            self.getConfigurationSection(), 'camera')
        cameraModel = self.configuration.deviceModel(cameraDeviceSection)
        if cameraModel == 'simulatedPyramidWfsCamera':
            self._createSimulatedPyramidWfsCamera(cameraDeviceSection)
        elif cameraModel == 'simulatedAuxiliaryCamera':
            self._createSimulatedAuxiliaryCamera(cameraDeviceSection)
        elif cameraModel == 'avt':
            self._createAvtCamera(cameraDeviceSection)
        else:
            raise KeyError('Unsupported camera model %s' % cameraModel)

    def _createSimulatedPyramidWfsCamera(self, cameraDeviceSection):
        cameraName = self.configuration.deviceName(cameraDeviceSection)
        self._camera = SimulatedPyramidWfsCamera(cameraName)
        self._setBinning(cameraDeviceSection)

    def _createSimulatedAuxiliaryCamera(self, cameraDeviceSection):
        cameraName = self.configuration.deviceName(cameraDeviceSection)
        self._camera = SimulatedAuxiliaryCamera(cameraName)
        self._setBinning(cameraDeviceSection)

    def _createAvtCamera(self, cameraDeviceSection):
        from pysilico_server.devices.avtCamera import AvtCamera
        from vimba import Vimba
        ipAddress = self.configuration.getValue(cameraDeviceSection,
                                                'ip_address')
        streamBytesPerSecond = self.configuration.getValue(
            cameraDeviceSection, 'streambytespersecond', getint=True)
        cameraName = self.configuration.deviceName(cameraDeviceSection)
        with Vimba.get_instance() as v:
            self._vimbacamera = v.get_camera_by_id(ipAddress)
        self._camera = AvtCamera(self._vimbacamera, cameraName)
        self._camera.setStreamBytesPerSecond(streamBytesPerSecond)
        self._setBinning(cameraDeviceSection)
        self._use_vimba_wrapper = True

    def _setBinning(self, cameraDeviceSection):
        try:
            binning = self.configuration.getValue(
                cameraDeviceSection, 'binning', getint=True)
            self._camera.setBinning(binning)
        except Exception:
            self._logger.warn(
                "binning not set (not specified in configuration?)")

    def _replyPort(self):
        return self.configuration.replyPort(self.getConfigurationSection())

    def _publisherPort(self):
        return self.configuration.publisherPort(self.getConfigurationSection())

    def _statusPort(self):
        return self.configuration.statusPort(self.getConfigurationSection())

    def _setUp(self):
        self._logger = Logger.of("Camera Controller runner")

        self._zmqPorts = ZmqPorts.fromConfiguration(
            self.configuration, self.getConfigurationSection())
        self._replySocket = self.rpc().replySocket(
            self._zmqPorts.SERVER_REPLY_PORT)
        self._publishSocket = self.rpc().publisherSocket(
            self._zmqPorts.SERVER_PUBLISHER_PORT, hwm=100)
        self._statusSocket = self.rpc().publisherSocket(
            self._zmqPorts.SERVER_STATUS_PORT, hwm=1)
        self._displaySocket = self.rpc().publisherSocket(
            self._zmqPorts.SERVER_DISPLAY_PORT, hwm=1)

        self._createCameraDevice()
        # self._camera.startAcquisition()

        self._controller = CameraController(
            self.name,
            self._zmqPorts,
            self._camera,
            self._replySocket,
            self._publishSocket,
            self._statusSocket,
            self._displaySocket,
            self.rpc())

    @ContextWrapper()
    def _runLoop(self):
        self._logRunning()

        self._camera.startAcquisition()
        FaultTolerantControlLoop(
            self._controller,
            Logger.of("Camera Controller control loop"),
            time,
            0.02).start()
        self._logger.notice("Terminated")

    @override
    def run(self):
        self._setUp()
        self._runLoop()
        return os.EX_OK

    @override
    def terminate(self, signal, frame):
        self._controller.terminate()



import os
from pysilico_server.utils.constants import Constants
from pysilico_server.utils.process_startup_helper import ProcessStartUpHelper
from plico.utils.starter_script_creator_base import StarterScriptCreatorBase


__version__= "$Id: starter_script_creator.py 31 2018-01-27 10:47:29Z lbusoni $"


class StarterScriptCreator(StarterScriptCreatorBase):

    def __init__(self):
        StarterScriptCreatorBase.__init__(self)


    def installExecutables(self):
        psh= ProcessStartUpHelper()

        self._createAStarterScript(
            os.path.join(self._binDir, Constants.START_PROCESS_NAME),
            psh.processProcessMonitorStartUpScriptPath(),
            Constants.PROCESS_MONITOR_CONFIG_SECTION
        )
        self._createAStarterScript(
            os.path.join(self._binDir,
                         Constants.SERVER_1_PROCESS_NAME),
            psh.deformableMirrorStartUpScriptPath(),
            Constants.SERVER_1_CONFIG_SECTION
        )
        self._createAStarterScript(
            os.path.join(self._binDir,
                         Constants.SERVER_2_PROCESS_NAME),
            psh.deformableMirrorStartUpScriptPath(),
            Constants.SERVER_2_CONFIG_SECTION
        )
        self._createAStarterScript(
            os.path.join(self._binDir, Constants.KILL_ALL_PROCESS_NAME),
            psh.killAllProcessesStartUpScriptPath(),
            'foo'
        )
        self._createAStarterScript(
            os.path.join(self._binDir, Constants.STOP_PROCESS_NAME),
            psh.processProcessMonitorStopScriptPath(),
            'not used'
        )



#!/usr/bin/env python
import unittest
import numpy as np
import logging
from test.test_helper import Poller, ExecutionProbe
from plico.utils.logger import Logger
from pysilico_server.devices.simulated_camera import \
    SimulatedPyramidWfsCamera

__version__ = "$Id: simulated_camera_test.py 271 2017-04-28 16:49:02Z lbusoni $"


class SimulatedPyramidWfsCameraTest(unittest.TestCase):

    def setUp(self):
        self._setUpLogging()
        self._camera = SimulatedPyramidWfsCamera()

    def tearDown(self):
        self._camera.stopAcquisition()
        self._camera.deinitialize()

    def _setUpLogging(self):
        FORMAT = '%(asctime)s %(levelname)s %(message)s'
        logging.basicConfig(level=logging.DEBUG, format=FORMAT)
        self._logger = Logger.of(self.__class__.__name__)

    def testReadFrameDimensionsAndType(self):
        self._camera.setBinning(1)
        f = self._camera.readFrame()
        self.assertEqual(f.shape, (self._camera.SENSOR_H,
                                   self._camera.SENSOR_W))
        self.assertEqual(f.dtype, self._camera.DTYPE)

    def testSetRadiusAndNoise(self):
        self._camera.setNoiseInCount(100.)
        self._camera.setPupilsRadiusInUnbinnedPixels(100)
        _ = self._camera.readFrame()

    def testFluxIsProportionalToExposureTime(self):
        self._camera.setNoiseInCount(0)
        self._camera.setExposureTime(1.0)
        ima1ms = self._camera.readFrame()
        self._camera.setExposureTime(2.5)
        ima2ms = self._camera.readFrame()
        self.assertAlmostEqual(2.5 * ima1ms.max(),
                               ima2ms.max(),
                               delta=3)

    def testFrameIsSaturatedIfFluxIsTooHigh(self):
        self._camera.setTotalFluxPerMilliSecond(1e9)
        frame = self._camera.readFrame()
        self.assertEqual(self._camera.MAX_VALUE,
                         frame.max())

    def testChangePupilsCenter(self):
        self._camera.setNoiseInCount(0)
        self._camera.setExposureTime(1.0)
        cp = 0.5 * np.array([self._camera.SENSOR_H,
                           self._camera.SENSOR_W])
        d = 0.1 * self._camera.SENSOR_H
        centers = np.zeros((4, 2))
        centers[0,:] = [cp[0] - d, cp[1] - d]
        centers[1,:] = [cp[0] + d, cp[1] - d]
        centers[2,:] = [cp[0] - d, cp[1] + d]
        centers[3,:] = [cp[0] + d, cp[1] + d]
        self._camera.setPupilsCenterInUnbinnedPixels(centers)
        self._camera.readFrame()

    def testWavefrontFromZernikeVector(self):
        tipPtV2Rms = 0.25
        ttCoeff = np.array([-1, 10]) * 1e-6
        self._camera.setNoiseInCount(0)
        self._camera.setScaleInMeterPerPixel(5e-6)
        self._camera.setSlopeSaturationInRadians(100e-3)
        self._camera.setWavefrontFromZernikeVector(ttCoeff)
        self._camera.setExposureTime(10)
        frame = self._camera.readFrame().astype(float)
        total = frame.sum()
        (sy, sx) = frame.shape
        dx = frame[:, -sx // 2:].sum() - frame[:, 0:sx // 2].sum()
        dy = frame[-sy // 2:,:].sum() - frame[0:sy // 2,:].sum()
        ttFromFlux = np.sin(np.pi / 2 * np.array([dx / total, dy / total])) * \
            self._camera.getSlopeSaturationInRadians() * \
            self._camera.getScaleInMeterPerPixel() * \
            self._camera.getPupilsRadiusInUnbinnedPixels() * 2 * \
            tipPtV2Rms
        self.assertTrue(np.allclose(ttCoeff, ttFromFlux, rtol=0.05),
                        "Wanted %s, got %s" % (ttCoeff, ttFromFlux))

    def _callback(self, frame):
        print ('callback %d' % frame.counter())
        self._lastFrameCounter = frame.counter()

    def testCallbackStartAndStop(self):
        print('testcallback')
        self._lastFrameCounter = 0
        self._camera.registerCallback(self._callback)

        def _frameCounterIncreased(self, originalFrameCounter):
            if self._lastFrameCounter <= originalFrameCounter:
                raise Exception("last %d - original %d" % (
                    self._lastFrameCounter, originalFrameCounter))

        self._camera.startAcquisition()
        Poller(2).check(ExecutionProbe(_frameCounterIncreased, '', self, 0))
        self._camera.stopAcquisition()
        currentFrameCnt = self._lastFrameCounter
        self.assertRaises(
            RuntimeError,
            Poller(2).check,
            ExecutionProbe(_frameCounterIncreased, '', self, currentFrameCnt)
        )


if __name__ == "__main__":
    unittest.main()
